/*
 * Diffraction at sub-nucleon scale
 * Dipole amplitude for a IPglasma nucleus
 * Heikki Mäntysaari <mantysaari@bnl.gov>, 2015
 */

#ifndef diffractivecrosssection_hpp
#define diffractivecrosssection_hpp

#include <string>
#include <vector>
#include <ipglasma.hpp>
#include <wilsonline.hpp>

class DiffractiveCrossSection {
public:
    DiffractiveCrossSection(std::string file);
    ~DiffractiveCrossSection();

    double DistanceToOrigin(int q[2]);
    std::complex<double> ComplexAmplitude(double xpom, int q1[2], int q2[2]);
    std::complex<double> ComplexAmplitude(double xpom, WilsonLine quark, int q2[2]);
    std::complex<double> SumAmplitudes();

    struct quark_flavor{
        double mass;
        double e_charge;
    };

    double PsiGammaqqbarLongitudinal(double r, double z, double Q, const quark_flavor &quark);

    IPGlasma* GetIPGlasma() { return ipglasma; }

private:
    
    IPGlasma *ipglasma;
    //WilsonLine& IPGlasma::GetWilsonLine(double x, double y);
    //WilsonLine& IPGlasma::GetWilsonLine(int i);
    //int IPGlasma::WilsonLineCoordinate(int indx, int indy);
    //double IPGlasma::X(int ix);
    //double IPGlasma::Y(int iy);
    //std::vector< double > xcoords;
    //std::vector< double > ycoords;
    //std::vector< WilsonLine  > wilsonlines;

};

#endif /* diffractivecrosssection_hpp */
