#include <iostream>
#include <string>
#include "ipglasma.hpp"
#include <cmath>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <omp.h>

#include "diffractivecrosssection.hpp"

using namespace std;

int printDipoleAmplitudes(string fname, DiffractiveCrossSection & diffxs)
{
    //ofstream output0;
    //Initialize params
    double q1[2] = {0,0};
    double q2[2] = {0,0};
    double r = 0;
    double dipoleAmp = 0;
    //TODO: set number of different rotations for dipole!
    int N = 100;

    cout << fixed << setprecision(8);
    
    cout << "# Dipole amplitudes for b=0"<< endl;
    cout << "# " << N << " rotations " << endl;
    cout << "# Filename " << fname << endl;
    cout << "# -------------------------"<< endl;
    cout << "# r, D_p(r,0)" << endl;

    // loop to calculate dipole amplitude at radii r=j*10^i
    for (int i=-4; i < 2; i += 1){
        cout <<""<< endl;

        // use index j to get radius
        for (double j=1; j < 10; j += 1){
            r = j*pow(10, i);
            // go through different orientations of the dipole via rotations
            for (double k=0; k < N; k += 1){
            // inside cos/sin is k*1/N*2*3.141592
                q1[0] = 0.5*r*cos(k*1./N*2*3.141592);
                q1[1] = 0.5*r*sin(k*1./N*2*3.141592);
                q2[0] = -q1[0];
                q2[1] = -q1[1];
    
                dipoleAmp += diffxs.GetIPGlasma()->Amplitude(0, q1, q2);
            }

            dipoleAmp = dipoleAmp/N;
            cout << r << ", " << dipoleAmp << endl;
            dipoleAmp = 0;
        }
    }

    return 0; 
}


int main(int argc, char* argv[])
{
    //TODO if num_threads > 1 cout << "using omp";

    if (argc != 2){
        cerr << "# Usage: " << argv[0] << " filename 0(integrate)/1(print dipole amplitudes)" << endl;
    }
    
    string fname = argv[1];
    //IPGlasma ipglasma(fname, 0, BINARY);
    DiffractiveCrossSection cross_section(argv[1]);

    if (argv[2]){ 
        printDipoleAmplitudes(fname, cross_section);
    }

    
    else{ 
        std::complex<double> sum = cross_section.SumAmplitudes();
        // Save results
        //std::filesystem::create_directory("output_"+fname);
        ofstream outfile("da_sum_ +" + fname + ".txt");
        outfile << sum << endl;
	outfile.close();
    }

    return 0;
}   
